/**********************************************************************/
//
// FILE:    drvrotmat.cpp
//          Partial derivatives of the rotation matrix functions.
// CREATED: 15 Dec 04  (ds)
// CHANGES: see include file
// REFS:    dito 
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include "drotmatrix3.h"
using namespace GenLib2;

namespace DEP
{

// partial derivatives of the rotation matrix

void derivs_opk_fix_cw(double o, double p, double k,
DRotMatrix3& dR_om, DRotMatrix3& dR_ph, DRotMatrix3& dR_ka)
{
	double so = sin(o);
	double sp = sin(p);
	double sk = sin(k);
	
	double co = cos(o);
	double cp = cos(p);
	double ck = cos(k);
	
	dR_om(0,0) =  0.;
	dR_om(0,1) = -sk * so + ck * sp * co;
	dR_om(0,2) =  sk * co + ck * sp * so;

	dR_om(1,0) =  0.;
	dR_om(1,1) = -ck * so - sk * sp * co;
	dR_om(1,2) =  ck * co - sk * sp * so;

	dR_om(2,0) =  0.;
	dR_om(2,1) = -cp * co;
	dR_om(2,2) = -cp * so;

	dR_ph(0,0) = -ck * sp;
	dR_ph(0,1) =  ck * cp * so;
	dR_ph(0,2) = -ck * cp * co;

	dR_ph(1,0) =  sk * sp;
	dR_ph(1,1) = -sk * cp * so;
	dR_ph(1,2) =  sk * cp * co;

	dR_ph(2,0) =  cp;
	dR_ph(2,1) =  sp * so;
	dR_ph(2,2) = -sp * co;

	dR_ka(0,0) = -sk * cp;
	dR_ka(0,1) =  ck * co - sk * sp * so;
	dR_ka(0,2) =  ck * so + sk * sp * co;

	dR_ka(1,0) = -ck * cp;
	dR_ka(1,1) = -sk * co - ck * sp * so;
	dR_ka(1,2) = -sk * so + ck * sp * co;

	dR_ka(2,0) =  0.;
	dR_ka(2,1) =  0.;
	dR_ka(2,2) =  0.;
}


// partial derivatives of the rotation matrix

void derivs_opk_fix_ccw(double o, double p, double k,
DRotMatrix3& dR_om, DRotMatrix3& dR_ph, DRotMatrix3& dR_ka)
{
	double so = sin(o);
	double sp = sin(p);
	double sk = sin(k);
	
	double co = cos(o);
	double cp = cos(p);
	double ck = cos(k);
	
	dR_om(0,0) =  0.;
	dR_om(0,1) =  sk * so + ck * sp * co;
	dR_om(0,2) =  sk * co - ck * sp * so;

	dR_om(1,0) =  0.;
	dR_om(1,1) = -ck * so + sk * sp * co;
	dR_om(1,2) = -ck * co - sk * sp * so;

	dR_om(2,0) =  0.;
	dR_om(2,1) =  cp * co;
	dR_om(2,2) = -cp * so;

	dR_ph(0,0) = -ck * sp;
	dR_ph(0,1) =  ck * cp * so;
	dR_ph(0,2) =  ck * cp * co;

	dR_ph(1,0) = -sk * sp;
	dR_ph(1,1) =  sk * cp * so;
	dR_ph(1,2) =  sk * cp * co;

	dR_ph(2,0) = -cp;
	dR_ph(2,1) = -sp * so;
	dR_ph(2,2) = -sp * co;

	dR_ka(0,0) = -sk * cp;
	dR_ka(0,1) = -ck * co - sk * sp * so;
	dR_ka(0,2) =  ck * so - sk * sp * co;

	dR_ka(1,0) =  ck * cp;
	dR_ka(1,1) = -sk * co + ck * sp * so;
	dR_ka(1,2) =  sk * so + ck * sp * co;

	dR_ka(2,0) =  0.;
	dR_ka(2,1) =  0.;
	dR_ka(2,2) =  0.;
}


// partial derivatives of the rotation matrix

void derivs_opk_rot_ccw(double o, double p, double k,
DRotMatrix3& dR_om, DRotMatrix3& dR_ph, DRotMatrix3& dR_ka)
{
	double so = sin(o);
	double sp = sin(p);
	double sk = sin(k);
	
	double co = cos(o);
	double cp = cos(p);
	double ck = cos(k);
	
	dR_om(0,0) =  0.;
	dR_om(0,1) =  0.;
	dR_om(0,2) =  0.;

	dR_om(1,0) = -so * sk + ck * sp * co;
	dR_om(1,1) = -ck * so - sk * sp * co;
	dR_om(1,2) = -cp * co;

	dR_om(2,0) =  ck * so * sp + sk * co;
	dR_om(2,1) = -so * sk * sp + ck * co;
	dR_om(2,2) = -cp * so;

	dR_ph(0,0) = -ck * sp;
	dR_ph(0,1) =  sp * sk;
	dR_ph(0,2) =  cp;

	dR_ph(1,0) =  ck * cp * so;
	dR_ph(1,1) = -sk * cp * so;
	dR_ph(1,2) =  sp * so;

	dR_ph(2,0) = -ck * co * cp;
	dR_ph(2,1) =  co * sk * cp;
	dR_ph(2,2) = -sp * co;

	dR_ka(0,0) = -sk * cp;
	dR_ka(0,1) = -cp * ck;
	dR_ka(0,2) =  0.;

	dR_ka(1,0) =  co * ck - sk * sp * so;
	dR_ka(1,1) = -sk * co - ck * sp * so;
	dR_ka(1,2) =  0.;

	dR_ka(2,0) =  sk * co * sp + ck * so;
	dR_ka(2,1) =  co * ck * sp - sk * so;
	dR_ka(2,2) =  0.;
}


// partial derivatives of the rotation matrix

void derivs_pok_rot_ccw(double p, double o, double k,
DRotMatrix3& dR_ph, DRotMatrix3& dR_om, DRotMatrix3& dR_ka)
{
	double so = sin(o);
	double sp = sin(p);
	double sk = sin(k);
	
	double co = cos(o);
	double cp = cos(p);
	double ck = cos(k);
	
	dR_ph(0,0) = -ck * sp + sk * cp * so;
	dR_ph(0,1) =  sp * sk + ck * cp * so;
	dR_ph(0,2) =  co * cp;

	dR_ph(1,0) =  0.;
	dR_ph(1,1) =  0.;
	dR_ph(1,2) =  0.;

	dR_ph(2,0) = -ck * cp - sp * sk * so;
	dR_ph(2,1) =  sk * cp - ck * sp * so;
	dR_ph(2,2) = -sp * co; 
	
	dR_om(0,0) =  sk * sp * co;
	dR_om(0,1) =  ck * sp * co;
	dR_om(0,2) = -so * sp;

	dR_om(1,0) = -so * sk;
	dR_om(1,1) = -ck * so;
	dR_om(1,2) = -co;

	dR_om(2,0) =  cp * sk * co;
	dR_om(2,1) =  ck * cp * co;
	dR_om(2,2) = -cp * so;

	dR_ka(0,0) = -sk * cp + ck * sp * so;
	dR_ka(0,1) = -cp * ck - sk * sp * so;
	dR_ka(0,2) =  0.;

	dR_ka(1,0) =  co * ck;
	dR_ka(1,1) = -sk * co;
	dR_ka(1,2) =  0.;

	dR_ka(2,0) =  sk * sp + cp * ck * so;
	dR_ka(2,1) =  ck * sp - sk * cp * so;
	dR_ka(2,2) =  0.;
}



// partial derivatives of the rotation matrix

void derivs_australis(double az, double el, double ro,
DRotMatrix3& dR_az, DRotMatrix3& dR_el, DRotMatrix3& dR_ro)
{
	double sa = sin(az);
	double se = sin(el);
	double sr = sin(ro);
	
	double ca = cos(az);
	double ce = cos(el);
	double cr = cos(ro);
	
	dR_az(0,0) = -sa * cr + ca * se * sr;
	dR_az(0,1) =  sa * sr + ca * se * cr;
	dR_az(0,2) =  ca * ce;

	dR_az(1,0) =  ca * cr + sa * se * sr;
	dR_az(1,1) = -ca * sr + sa * se * cr;
	dR_az(1,2) =  sa * ce;
 
	dR_az(2,0) =  0.;
	dR_az(2,1) =  0.;
	dR_az(2,2) =  0.; 
	
	dR_el(0,0) =  sa * ce * sr;
	dR_el(0,1) =  sa * ce * cr;
	dR_el(0,2) = -sa * se;

	dR_el(1,0) = -ca * ce * sr;
	dR_el(1,1) = -ca * ce * cr;
	dR_el(1,2) =  ca * se;

	dR_el(2,0) = -se * sr;
	dR_el(2,1) = -se * cr;
	dR_el(2,2) = -ce;

	dR_ro(0,0) = -ca * sr + sa * se * cr;
	dR_ro(0,1) = -ca * cr - sa * se * sr;
	dR_ro(0,2) =  0.;

	dR_ro(1,0) = -sa * sr - ca * se * cr;
	dR_ro(1,1) = -sa * cr + ca * se * sr;
	dR_ro(1,2) =  0.;

	dR_ro(2,0) =  ce * cr;
	dR_ro(2,1) = -ce * sr;
	dR_ro(2,2) =  0.;
}


// partial derivatives of the rotation matrix

void derivs_quaternion(double q0, double q1, double q2, double q3,
DRotMatrix3& dR_q0, DRotMatrix3& dR_q1, DRotMatrix3& dR_q2, DRotMatrix3& dR_q3)
{
	//dR_q0(0,0) =  2. * q0;
	dR_q0(0,0) = 0.;
	dR_q0(0,1) = -2. * q3;
	dR_q0(0,2) =  2. * q2;

	dR_q0(1,0) =  2. * q3;
	//dR_q0(1,1) =  2. * q0;
	dR_q0(1,1) =  0.;
	dR_q0(1,2) = -2. * q1;

	dR_q0(2,0) = -2. * q2;
	dR_q0(2,1) =  2. * q1;
	//dR_q0(2,2) =  2. * q0;
	dR_q0(2,2) =  0.;

	//dR_q1(0,0) =  2. * q1;
	dR_q1(0,0) =  0.;
	dR_q1(0,1) =  2. * q2;
	dR_q1(0,2) =  2. * q3;

	dR_q1(1,0) =  2. * q2;
	//dR_q1(1,1) = -2. * q1;
	dR_q1(1,1) = -4. * q1;
	dR_q1(1,2) = -2. * q0;

	dR_q1(2,0) =  2. * q3;
	dR_q1(2,1) =  2. * q0;
	//dR_q1(2,2) = -2. * q1;
	dR_q1(2,2) = -4. * q1;
	
	//dR_q2(0,0) = -2. * q2;
	dR_q2(0,0) = -4. * q2;
	dR_q2(0,1) =  2. * q1;
	dR_q2(0,2) =  2. * q0;

	dR_q2(1,0) =  2. * q1;
	//dR_q2(1,1) =  2. * q2;
	dR_q2(1,1) =  0.;
	dR_q2(1,2) =  2. * q3;

	dR_q2(2,0) = -2. * q0;
	dR_q2(2,1) =  2. * q3;
	//dR_q2(2,2) = -2. * q2;
	dR_q2(2,2) = -4. * q2;
	
	//dR_q3(0,0) = -2. * q3;
	dR_q3(0,0) = -4. * q3;
	dR_q3(0,1) = -2. * q0;
	dR_q3(0,2) =  2. * q1;

	dR_q3(1,0) =  2. * q0;
	//dR_q3(1,1) = -2. * q3;
	dR_q3(1,1) = -4. * q3;
	dR_q3(1,2) =  2. * q2;

	dR_q3(2,0) =  2. * q1;
	dR_q3(2,1) =  2. * q2;
	//dR_q3(2,2) =  2. * q3;
	dR_q3(2,2) =  0.;
}


// partial derivatives of the rotation matrix

void derivs_rodrigues(double a, double b, double c,
DRotMatrix3& dR_a, DRotMatrix3& dR_b, DRotMatrix3& dR_c)
{
	double v = 1. + (a * a + b * b + c * c) / 4.;
	
	DRotMatrix3 U;
	U.set_rodrigues_matrix(a, b, c);
	
	DRotMatrix3 dU_a;
	dU_a(0,0) =  0.5 * a;
	dU_a(0,1) =  0.5 * b;
	dU_a(0,2) =  0.5 * c;

	dU_a(1,0) =  0.5 * b;
	dU_a(1,1) = -0.5 * a;
	dU_a(1,2) = -1.;

	dU_a(2,0) =  0.5 * c;
	dU_a(2,1) =  1.;
	dU_a(2,2) = -0.5 * a;

	DRotMatrix3 dU_b;
	dU_b(0,0) =  0.5 * b;
	dU_b(0,1) =  0.5 * a;
	dU_b(0,2) =  1.;

	dU_b(1,0) =  0.5 * a;
	dU_b(1,1) =  0.5 * b;
	dU_b(1,2) =  0.5 * c;

	dU_b(2,0) = -1.;
	dU_b(2,1) =  0.5 * c;
	dU_b(2,2) = -0.5 * b;

	DRotMatrix3 dU_c;
	dU_c(0,0) = -0.5 * c;
	dU_c(0,1) =  1.;
	dU_c(0,2) =  0.5 * a;

	dU_c(1,0) =  1.;
	dU_c(1,1) = -0.5 * c;
	dU_c(1,2) =  0.5 * b;

	dU_c(2,0) =  0.5 * a;
	dU_c(2,1) =  0.5 * b;
	dU_c(2,2) =  0.5 * c;

	double dv_a = 0.5 * a;
	double dv_b = 0.5 * b;
	double dv_c = 0.5 * c;

	double q = v * v;
	
	dR_a(0,0) = (dU_a(0,0) * v - U(0,0) * dv_a) / q;
	dR_a(0,1) = (dU_a(0,1) * v - U(0,1) * dv_a) / q;
	dR_a(0,2) = (dU_a(0,2) * v - U(0,2) * dv_a) / q;
		
	dR_a(1,0) = (dU_a(1,0) * v - U(1,0) * dv_a) / q;
	dR_a(1,1) = (dU_a(1,1) * v - U(1,1) * dv_a) / q;
	dR_a(1,2) = (dU_a(1,2) * v - U(1,2) * dv_a) / q;
		
	dR_a(2,0) = (dU_a(2,0) * v - U(2,0) * dv_a) / q;
	dR_a(2,1) = (dU_a(2,1) * v - U(2,1) * dv_a) / q;
	dR_a(2,2) = (dU_a(2,2) * v - U(2,2) * dv_a) / q;
	
	dR_b(0,0) = (dU_b(0,0) * v - U(0,0) * dv_b) / q;
	dR_b(0,1) = (dU_b(0,1) * v - U(0,1) * dv_b) / q;
	dR_b(0,2) = (dU_b(0,2) * v - U(0,2) * dv_b) / q;
		
	dR_b(1,0) = (dU_b(1,0) * v - U(1,0) * dv_b) / q;
	dR_b(1,1) = (dU_b(1,1) * v - U(1,1) * dv_b) / q;
	dR_b(1,2) = (dU_b(1,2) * v - U(1,2) * dv_b) / q;
		
	dR_b(2,0) = (dU_b(2,0) * v - U(2,0) * dv_b) / q;
	dR_b(2,1) = (dU_b(2,1) * v - U(2,1) * dv_b) / q;
	dR_b(2,2) = (dU_b(2,2) * v - U(2,2) * dv_b) / q;
		
	dR_c(0,0) = (dU_c(0,0) * v - U(0,0) * dv_c) / q;
	dR_c(0,1) = (dU_c(0,1) * v - U(0,1) * dv_c) / q;
	dR_c(0,2) = (dU_c(0,2) * v - U(0,2) * dv_c) / q;
		
	dR_c(1,0) = (dU_c(1,0) * v - U(1,0) * dv_c) / q;
	dR_c(1,1) = (dU_c(1,1) * v - U(1,1) * dv_c) / q;
	dR_c(1,2) = (dU_c(1,2) * v - U(1,2) * dv_c) / q;
		
	dR_c(2,0) = (dU_c(2,0) * v - U(2,0) * dv_c) / q;
	dR_c(2,1) = (dU_c(2,1) * v - U(2,1) * dv_c) / q;
	dR_c(2,2) = (dU_c(2,2) * v - U(2,2) * dv_c) / q;
	}

} // namespace DEP
