/* $Id: eoquaternion.h,v 1.10 2007/10/11 13:18:28 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_EOQUATERNION_INCLUDED
#define DEP_EOQUATERNION_INCLUDED

#include "extori.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Exterior orientation using quaternion class.
 *  
 *  Parameters: 3 translations, 4 rotations
 *  Constraints: 1 between the rotation parameters
 *   
 *  @author Dirk Stallmann
 */
class EOQuaternion : public ExtOrientation
{
public:
	/// constructor
	EOQuaternion() : ExtOrientation(4) {}

	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		string key_str() const;

		/// get parameter name
		string uname(size_t i) const;

		/// get number of constraints
		size_t num_con() const { return 1; }

		/// add weighted constraints
		void add_constraints(Matrix<double>& atpa,
			Vector<double>& atpl, double& ltpl) const;
	//@}

	/**@name resolved pure virtual methods of class ExtOrientation */
	//@{
		/// calculate rotation parameters from rotation matrix
		void calc_rot_params(vector<double>& a, const DRotMatrix3& R) const
		{
			assert(a.size() == 4);
			R.get_quaternion(a[0], a[1], a[2], a[3]);
		}
		
		/// calculate rotation matrix
		void calc_rot_matrix(DRotMatrix3& R, const vector<double>& a) const
		{
			assert(a.size() == 4);
			R.set_quaternion(a[0], a[1], a[2], a[3]);
		}

		/// calculate partial derivatives of rotation parameters
		void calc_derivs_rot(DerivSub& deriv,
			double h, double u, double v, double w,
			const DPoint3& d, const vector<double>& a) const;

		/// calculate partial derivatives of rotation parameters
		void calc_derivs_rot(DerivSub& deriv,
			double h, double u, double v, double w,
			const DPoint3& d, const DRotMatrix3& M, const vector<double>& a) const;
	//@}
	
private:
	/// calculate partial derivatives of the rotation matrix
	void dr(double q0, double q1, double q2, double q3,
	DRotMatrix3& dR_q0, DRotMatrix3& dR_q1,
	DRotMatrix3& dR_q2, DRotMatrix3& dR_q3) const
	{
		derivs_quaternion(q0, q1, q2, q3, dR_q0, dR_q1, dR_q2, dR_q3);
	}
};


// non-member functions ...

} // namespace DEP

#endif
