/* $Id: imgpnt.cpp,v 1.4 2007/07/23 09:40:04 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "imgpnt.h"

namespace DEP
{

// read image point

bool ImagePoint::read(istream& is)
{
	is >> _id;
	if (!is) return false;

	is >> crds >> sdev;
	if (!is) return false;

	return true;
}


// non-member functions ...


// add (append or replace) image point

bool add_image_point(vector<ImagePoint*>& impts, ImagePoint* ip)
{
	assert(ip != 0);
	bool rtn = false;

	int i = find_image_point(impts, ip->_id);
	if (i < 0)
		{
		impts.push_back(ip);
		rtn = true;
		}
	else
		{
		ImagePoint* tmp = impts[i];
		impts[i] = ip;
		delete tmp;
		cout << "ATTENTION: Image point " << ip->_id
			<< " was replaced.\n";
		}

    return rtn;
}


// find image point by id, return index

int find_image_point(const vector<ImagePoint*>& impts, const string& id)
{
	int i = 0;

	vector<ImagePoint*>::const_iterator ip;
	for (ip = impts.begin(); ip != impts.end(); ++ip)
		{
		if (*ip != 0 && (*ip)->get_id() == id) return i;
		i++;
		}

	return -1;
}


// count number of image points

int count_image_points(const vector<ImagePoint*>& impts)
{
	int n = 0;

	vector<ImagePoint*>::const_iterator ip;
	for (ip = impts.begin(); ip != impts.end(); ++ip)
		{
		if (*ip != 0) n++;
		}

	return n;
}

} // namespace DEP
