/**********************************************************************/
//
// FILE:    imumis.h
//          IMU misalignment class.
// CREATED: 27 Feb 04  (ds)
// CHANGES:
// REFS:
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#ifndef DEP_IMUMIS_INCLUDED
#define DEP_IMUMIS_INCLUDED

#include "adjobj.h"
#include "drvrotmat.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{
class Image;
class DerivSub;

/** IMU misalignment class.
 *  Intermediate class for IMU misalignment classes.
 *  Don't use this class directly!
 *
 *  @author Dirk Stallmann
 */
class IMUMisalignment : public virtual AdjObject
{
public:
	/// constructor
	IMUMisalignment(size_t sz);
	
	/// virtual destructor
	virtual ~IMUMisalignment() {}
	
	/**@name read/write/print IMU misalignment data */
	//@{
		/// read
		virtual bool read(istream& is);

		/// write
		virtual void write(ostream& os) const;

		/// print
		virtual void print(ostream& os) const;
	//@}
	
	/**@name overridden methods of base class AdjObject
	 */
	//@{
		/// get adjustment object class name (long description)
		string class_str() const;
		
		/// set unknown parameter information (upari)
		bool set_upari();
	//@}
	
	/// attach to image
	bool attach(vector<Image*>& images);
	
	/**@name pure vitual methods */
	//@{
		/// calculate rotation parameters from rotation matrix
		virtual void calc_rot_params(vector<double>& a,
			const DRotMatrix3& M) const = 0;
		
		/// get rotation matrix
		virtual void get_rot_matrix(DRotMatrix3& M) const = 0;
	
		/// calculate derivatives
		virtual void calc_derivs(DerivSub& deriv,
			double h, double u, double v, double w,
			const DPoint3& d, const DRotMatrix3& R) const = 0;
	//@}
	
protected:
	/// image IDs that use this set
	vector<string>	_image_ids;

	/// rotation parameters min. 3 until max. 9
	vector<double>	_a;

	/// std. devs. of rotation parameters
	vector<double>	_sa;
};


// non-member functions ...

} // namespace DEP

#endif
