/**********************************************************************/
//
// FILE:    imuopkfix.h
//          IMU misalignment class using omega-phi-kappa angles
//          about fixed axes.
// CREATED: 01 Nov 04  (ds)
// CHANGES:
// REFS:
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#ifndef DEP_IMUOPKFIX_INCLUDED
#define DEP_IMUOPKFIX_INCLUDED

#include "imumis.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** IMU misalignment class.
 *  Rotation angles 
 *  about fxed axes.
 *
 *  @author Dirk Stallmann
 */
class IMUMisalignmentOPKFix : public IMUMisalignment
{
public:
	/// constructor
	IMUMisalignmentOPKFix() : IMUMisalignment(3) {};
	
	/**@name read/write/print IMU misalignment data */
	//@{
		/// read
		bool read(istream& is);

		/// write
		void write(ostream& os) const;

		/// print
		void print(ostream& os) const;
	//@}
	
	/**@name overridden base class members of AdjObject */
	//@{
		/// get type name
		string type_str() const;

		/// get key name
		string key_str() const;

		/// get parameter name
		string uname(size_t i) const;
	//@}
		
	/**@name overridden methods of base class AdjObject
	 */
	//@{
		/// print changes of unknowns
		virtual void print_changes(ostream& os, const Vector<double>& dx) const;

		/// print unknown parameters
		virtual void print_unknowns(ostream& os) const;
		
		/// print precision of unknowns
		virtual void print_precision(ostream& os) const;
	//@}
	
	/**@name overridden class members of IMUMisalignment */
	//@{
		/// calculate rotation parameters from rotation matrix
		void calc_rot_params(vector<double>& a,
		const DRotMatrix3& M) const
		{
			assert(a.size() == 3);
			M.get_angles_opk_fix(a[0], a[1], a[2]);
		}

		/// get rotation matrix from rotation parameters
		void get_rot_matrix(DRotMatrix3& M) const
		{
			assert(_a.size() == 3);
			M.set_angles_opk_fix(_a[0], _a[1], _a[2]);
		}
		
		/// calculate derivatives
		void calc_derivs(DerivSub& deriv,
			double h, double u, double v, double w,
			const DPoint3& d, const DRotMatrix3& R) const;
	//@}

protected:
	/// differential rotation matrices
	void dm(DRotMatrix3& dM_om, DRotMatrix3& dM_ph,
	DRotMatrix3& dM_ka) const
	{
		assert(_a.size() == 3);
		derivs_opk_fix_cw(_a[0], _a[1], _a[2], dM_om, dM_ph, dM_ka);
	}
};


// non-member functions ...

} // namespace DEP

#endif
