/* $Id: pbcam.h,v 1.8 2007/07/23 09:41:12 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_PBCAM_INCLUDED
#define DEP_PBCAM_INCLUDED

#include "ccdcam.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Pushbroom camera class.
 *  Implementation of an ''ideal'' pushbroom camera.
 *  Contains the inner orientation parameters of the imaging sensor.
 *  Assuming the CCD line is mounted width the line center at nominal
 *  position xo perpendicular to the x-axis in the focal plane.
 *  The scanning period per line or scan frequency is not defined
 *  here because the value might be variable for the camera!
 *
 *  @author Dirk Stallmann
 */
class PBCamera : public CCDCamera
{
public:
	/// constructor
	PBCamera();
	
	/**@name set/get member data */
	//@{
		/// get camera type
		virtual string type_str() const;
	
		/// get keyword name
		virtual string key_str() const;
	//@}

	/**@name sensor/image coordinates transformation
	 *  To convert between sensor coordinates (column or sample and
	 *  row or line) and image coordinates.
	 *  Attention: The line (y) component represents the exposure
	 *  time! So this component is unused respectivly unchanged.
	 */
	//@{
		/// sensor-to-image coordinates transformation
		virtual void sensor_to_image(const DPoint2& pix_c,
			DPoint2& img_c) const;

		/// image-to-sensor coordinates transformation
		virtual void image_to_sensor(const DPoint2& img_c,
			DPoint2& pix_c) const;
	//@}

	/**@name read/write/print camera parameters
	 */
		/// read camera parameters
		virtual bool read(istream& is);
	
		/// write camera parameters
		virtual void write(ostream& os) const;
		
		/// print camera parameters
		virtual void print(ostream& os) const;
	//@}

	/// Ground Sampling Distance (GSD) in sensor line direction
	double GSD(double hg) const;

private:
	/// nominal CCD line position
	double	_x0;
};

} // namespace DEP

#endif
