/**********************************************************************/
//
// FILE:    pbcamcal.h
//          Calibrated pushbroom camera class.
// CREATED: 19 Feb 05  (ds)
// CHANGES:
// REFS:
//
// (c) 2005 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#ifndef DEP_PBCAMCAL_INCLUDED
#define DEP_PBCAMCAL_INCLUDED

#include "ccdcam.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Calibrated pushbroom camera class.
 *  
 *
 *  @author Dirk Stallmann
 */
class PBCameraCalib : public CCDCamera
{
public:
	/// constructor
	PBCameraCalib();

	/// virtual destructor
	virtual ~PBCameraCalib();
	
	/**@name set/get member data */
	//@{
		/// get camera type
		virtual string type_str() const;
	
		/// get keyword name
		virtual string key_str() const;
	//@}
	
	/**@name read/write/print camera parameters
	 */
		/// read camera data
		virtual bool read(istream& is);
	
		/// write camera data
		virtual void write(ostream& os) const;
		
		/// print camera data
		virtual void print(ostream& os) const;
	//@}

	/**@name sensor/image coordinates transformation
	 *  To convert between sensor (pixel) and image coordinates
	 *  by lineare interpolation between neigbouring
	 *  calibrated pixel positions.
	 *  For pixel values outside the calibrated pixel positions
	 *  the image or pixel coordinates are extrapolated by
	 *  using the first and second or the second last and last
	 *  calibrated pixel positions.
	 *  Attention: The line/row (y) component represents the exposure
	 *  time! So this component is unused here. This value will not
	 *  changed by this function!
	 */
	//@{
		/// sensor-to-image coordinates transformation
		virtual void sensor_to_image(const DPoint2& pix_c,
			DPoint2& img_c) const;

		/// image-to-sensor coordinates transformation
		virtual void image_to_sensor(const DPoint2& img_c,
			DPoint2& pix_c) const;
	//@}

protected:
	/// calibrated pixel position in x
	double*		_xcal;

	/// calibrated pixel position in y
	double*		_ycal;
	
	/** locate in ordered table
	 *  assume elements increasing!
	 *  see Numerical Recipes in C, 2nd ed. page 117
	 */
	int locate(double xx[], double x) const;
};

} // namespace DEP

#endif
