/**********************************************************************/
//
// FILE:    pnpars.cpp
//          Photogrammetric network design parameters and options class.
// CREATED: 15 Jan 04  (ds)
// CHANGES: see include file
// REFS:    dito
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#include "pnpars.h"

namespace DEP
{

// constructor

PNParams::PNParams()
{
	_min_rays_point = 2;	// two rays per object point (default)
							// are useful for bundle adjustment,
							// for resection use 1 ray as limit,
							// for testfield calibration use 3
							// rays or more
	_min_rays_image = 3;	// two rays per image
	pr_freq_rays = false;
}


// read

bool PNParams::read(istream& is)
{
	is >> _min_rays_point;
	if (_min_rays_point < 1)
		{
		cout << "ERROR: number of rays must be larger than zero: "
			<< _min_rays_point << endl;
		return false;
		}

	is >> _min_rays_image;
	if (_min_rays_image < 1)
		{
		cout << "ERROR: number of rays must be larger than zero: "
			<< _min_rays_image << endl;
		return false;
		}

	is >> pr_freq_rays;
	
	return true;
}


// print

void PNParams::print(ostream& os) const
{
	os << "Photogrammetric network design parameters/options\n";
	os << "=================================================\n";
	os << "Minimal number of rays\n";
	os << "  - per object point               :  " << _min_rays_point << endl;
	os << "  - per image                      :  " << _min_rays_image << endl;
	os << endl;
	os << "Print abs. frequency of image rays :  "
		<< (pr_freq_rays ? "on" : "off") << endl;
	os << endl;
}

} // namespace DEP
