/**********************************************************************/
//
// FILE:    pnpars.h
//          Photogrammetric network design parameters and options class.
// CREATED: 15 Jan 04  (ds)
// CHANGES:
// REFS:
//
// (c) 2004 Dirk Stallmann (ds)/ IfP Uni Stuttgart
/**********************************************************************/

#ifndef DEP_PNPARS_INCLUDED
#define DEP_PNPARS_INCLUDED

#include "depdef.h"

using namespace std;
using namespace GenLib2;

namespace DEP
{

/** Photogrammetric network design parameters and options class.
 *  @author Dirk Stallmann
 */
class PNParams
{
	// allows direct access for derived adjustment objects
	friend class AdjObject;

public:
	/// constructor
	PNParams();

	/// virtual destructor
	virtual ~PNParams() {}

	/// read parameters
	bool read(istream& is);

	/// print parameters
	void print(ostream& os) const;

	/// get minimal number of rays per object point
	int min_rays_point() const { return _min_rays_point; }

	/// get minimal number of rays per image
	int min_rays_image() const { return _min_rays_image; }

public:
	/// minimal number of rays per object point
	int		_min_rays_point;

	/// minimal number of rays per image
	int		_min_rays_image;
	
	/// print absolute frequency of image rays
	bool	pr_freq_rays;
};

} // namespace DEP

#endif
