/* $Id: upari.h,v 1.4 2007/07/23 09:43:42 cvsroot Exp $ */

/* DGAP - General Analytical Positioning
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef DEP_UPARI_INCLUDED
#define DEP_UPARI_INCLUDED


namespace DEP
{

/// unknown parameter status
enum u_state
{
	/// unknown
	unk,
	
	/// observed
	obs,
	
	/// fixed
	fix,
	
	/// eliminated
	eli
};


/** Unknowns parameter information structure.
 *  Unknowns PARameter Information structure (UPARI). It contains pointers
 *  to the value of the parameters, as well as its oberved or eliminated
 *  value as well as the standard deviation in the case of observed
 *  and eliminated parameters.
 *  @author Dirk Stallmann
 */
struct Upari
{
	/// initial value of the parameter
	double	ival;

	/// pointer to parameter value
	double*	par;

	/** pointer to std.dev. of the observed or eliminated value
	 *  of the parameter
	 */
	double*	sdev;

	/** initial state of parameter, indicates wether parameter is:
	 *  unknown (unk), observed (obs) or eliminated (eli). The initial
	 *  state is defined by the given std. dev. of the unknown parameter.
	 *  The state may be changed only by the automatic elimination of
	 *  parameters during the iteration process from the states unknown
	 *  or observed to eliminated.
	 */
	u_state	state;

	/// weight of parameter in case of observed or eliminated status
	double	pii;
};


} // namespace DEP

#endif
